@@ -13,7 +13,6 @@ module SassSpec::CLI
       verbose: false,
       filter: "",
       limit: -1,
-      nuke: false,
       only_output_styles: []
     }
 
@@ -60,7 +59,7 @@ Make sure the command you provide prints to stdout.
         options[:engine_adapter] = ExecutableEngineAdapater.new(v)
       end
 
-      opts.on("-g", "--generate", "Run test(s) and generate expected output file(s).") do |v|
+      opts.on("-g", "--generate", "Run test(s) and generate expected output file(s).") do
         options[:generate] = true
       end
 
@@ -98,14 +97,6 @@ Make sure the command you provide prints to stdout.
         options[:spec_directory] = File.expand_path(spec_dir)
       end
 
-      opts.on("-s", "--skip", "Skip tests that fail to exit successfully") do
-        # Note: --skip is no longer necessary as this is now the default behavior, since we test for errors
-      end
-
-      opts.on("--nuke", "Write a new expected_output for every test from whichever engine we are using") do
-        options[:nuke] = true
-      end
-
       opts.on("--migrate", "Copy tests that fail and make them pass for the current version.") do
         options[:migrate] = true
       end
@@ -114,6 +105,10 @@ Make sure the command you provide prints to stdout.
         options[:silent] = true
       end
 
+      opts.on("--check-annotations", "Check if any test annotations are unecessary.") do
+        options[:check_annotations] = true
+      end
+
       opts.on("--interactive", "When a test fails, enter into a dialog for how to handle it.") do
         options[:interactive] = true
       end
